#!/bin/sh
# boot_ubuntu.sh - boot Ubuntu s390x image using hercules
# Geoff Carpenter - gcc@fargos.net

originDir=`dirname $0`
sudoCmd=""
mountISO=""
useScreen=0
noAuto=0
rcBootFile="${originDir}/hercules-ipl.rc"

defaultGateway=`netstat -rn | awk '/^0.0.0.0/ { print $2; exit; }'`
defaultHostname="ubuntu-z"
DNSserver="${defaultGateway}"
initialPW="initialpw"
defaultDomain=`dnsdomainname`

while test $# -gt 0
do
	case "${1}" in
	-h | -help | --help)
		printf "usage: %s [--screen] [{--iso isoFileName [--noauto] [--dns DNSserver] [--gw gatewayAddr] [--host hostName] [--domain domainName] [--pw installPassword]}]\n" "${0}" >&2
		printf "  --iso image = boot from CD image\n" >&2
		printf "  Image will be associated with a read-only loopback device and mounted,\n" >&2
		printf "  then IPLed using ubuntu.ins in the root directory\n" >&2
		printf " --noauto inhibits automatic responses to install dialog\n" >&2
		printf "  Default gateway and DNS server: %s\n" "${defaultGateway}" >&2
		printf "  Default hostname: %s\n" "${defaultHostname}" >&2
		printf "  Default domain: %s\n" "${defaultDomain}" >&2
		printf "  Default install password: %s\n" "${initialPW}" >&2
		exit 1
		;;
	--screen)
		useScreen=1
		;;
	--iso)
		mountISO="${2}"
		shift
		;;
	--noauto)
		noAuto=1
		;;
	--dns)
		DNSserver="${2}"
		shift
		;;
	--domain)
		defaultDomain="${2}"
		shift
		;;
	--gw)
		defaultGateway="${2}"
		shift
		;;
	--host)
		defaultHostname="${2}"
		shift
		;;
	--pw)
		initialPW="${2}"
		shift
		;;
	--pidfile)
		shift
		;;
	-*)
		printf "%s: unrecognized option: \"%s\"\n" "${0}" "${1}" >&2
		exit 1
		;;
	*)
		printf "%s: unrecognized argument: \"%s\"\n" "${0}" "${1}" >&2
		exit 1
		;;
	esac
	shift
done

userId=`id -u`

if test ${userId} -ne 0
then # not root
	printf "%s: using sudo to gain root permission to configure network\n" "${0}"
	sudoCmd="sudo"
fi

# clear IP tables, setup NAT
${sudoCmd} ${originDir}/setupCTCnetwork
rc=$?
if test ${rc} -ne 0
then
	printf "%s: could not configure networking\n" "${0}" >&2
	exit 1
fi

${sudoCmd} ${originDir}/setupSSHforwarding

# start hercules

if test -n "${mountISO}"
then # boot from mounted ISO
	useScreen=0 # disable
	mountPoint="/tmp/s390x.iso"
	useLoopDevice=`${sudoCmd} losetup -r --find --show ${mountISO}`
	${sudoCmd} mkdir -p ${mountPoint}
	${sudoCmd} mount -r ${useLoopDevice} ${mountPoint}
	iplInstructions=`ls ${mountPoint}/ubuntu.ins`
	if test -z "${iplInstructions}"
	then
		printf "%s: could not find IPL instructions, expected ubuntu.ins under %s\n" "${0}" "${mountPoint}" >&2
		exit 1
	fi
	rcBootFile="/tmp/iplFromISO.rc"
# ipl instructions
# 1 = use CTC
# 1 = set read device
# 2 = set write device
# 2 = set protocol to Linux
# 2 = set network configuration manually
# 10.1.1.2 = local IP address
# 255.255.255.0 = network mask
# ${defaultGateway} = gateway
# ${defaultDNS} = nameserver
# ${defaultHostname} = hostname
# ${defaultDomain} = domain name
# initialpw = install password
# initialpw = install password
	echo ipl ${iplInstructions} > ${rcBootFile}
	if test ${noAuto} -eq 0
	then
		cat >> ${rcBootFile} <<endIPL
.1
.1
.2
.2
.2
.10.1.1.2
.255.255.255.0
.${defaultGateway}
.${defaultHostname}
.${DNSserver}
.${defaultDomain}
.${initialPW}
.${initialPW}
endIPL
	fi
	trap "rm -f ${rcBootFile}" EXIT
fi

if test ${useScreen} -eq 1
then
	screen -D -m -S 'hercules zSeries Ubuntu' -t 'zeries Ubuntu' \
	hercules -f ${originDir}/hercules-ubuntu.cfg --rc ${rcBootFile}
	sleep 2
else
	hercules -f ${originDir}/hercules-ubuntu.cfg --rc ${rcBootFile}
	if test -n "${mountISO}"
	then # cleanup and unmount ISO image
		echo umount ${mountPoint}
		umount ${mountPoint}
		echo losetup -d ${useLoopDevice}
		losetup -d ${useLoopDevice}
	fi
fi
